package PerlCAD::Command;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use strict;
use Data::DumpEx;

###----------------------------------------------------------------###

register_action({
  name      => 'draw_line',
  accel     => 'Ctrl+L',
  menutext  => '&Line',
  tooltip   => 'Draw A Line',
  whatsthis => '<img src="draw_line"> <b>Draw Line</b><br/>Draw a line between two specified points',
  slot      => 'draw_line_slot',
  method    => sub {
    this->slot_process_line('LINE');
  },
});

register_command({
  command => 'LINE',
  aliases => ['L'],
  method  => \&draw_line,
  requires => [{
    type => 'POINT',
    msg  => 'Specify first point',
    default_to_last => 1,
  }, {
    type => 'POINT',
    msg  => 'Specify next point',
    draw => sub {
      my $state = shift;
      my $p1    = shift;
      my $p2    = shift;
      return [['line', $p1, $p2]];
    },
  }],
  multiple => sub { this->get_state->{queue} = [ this->last_point ] },
});

###----------------------------------------------------------------###

sub draw_line {
  my $info  = shift;
  my $p1    = shift;
  my $p2    = shift;
  my $block = undef;
  my ($ltype, $color, $width) = @{ this->parent->get_line_info };
  my $layer = 'default';
  this->parent->doc->db->add_line($block, $ltype, $layer, $color, $width, $p1, $p2);
}

1;
