package PerlCAD::Command;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use strict;
use Data::DumpEx;

###----------------------------------------------------------------###

register_action({
  name      => 'draw_rect',
  menutext  => '&Rectangle',
  tooltip   => 'Draw A Rectangle',
  whatsthis => '<img src="draw_rect"> <b>Draw Rectangle</b><br/>Draw a rectangle specified by two points',
  slot      => 'draw_rect_slot',
  method    => sub {
    this->slot_process_line('RECTANG');
  },
});

register_command({
  command => 'RECTANG',
  aliases => ['RECT'],
  method  => \&draw_rect,
  requires => [{
    type => 'POINT',
    msg  => 'Please enter first corner of rectangle',
  }, {
    type => 'POINT',
    msg  => 'Please enter second corner of rectangle',
    draw => \&draw_rect_lines,
  }],
});

###----------------------------------------------------------------###

sub draw_rect_lines {
  my $state = shift;
  my $p1    = shift;
  my $p2    = shift;
  $_->[2] ||= 0 foreach $p1, $p2;
  die "Must be planar" if $p1->[2] != $p2->[2];
  return [['line', $p1, [$p1->[0], $p2->[1], $p1->[2]]],
          ['line', $p2, [$p1->[0], $p2->[1], $p1->[2]]],
          ['line', $p2, [$p2->[0], $p1->[1], $p1->[2]]],
          ['line', $p1, [$p2->[0], $p1->[1], $p1->[2]]],
          ];
}


sub draw_rect {
  my $lines = &draw_rect_lines;
  my $block = undef;
  my $layer = 'default';
  my ($ltype, $color, $width) = @{ this->parent->get_line_info };
  foreach (@$lines) {
    shift @$_;
    this->parent->doc->db->add_line($block, $ltype, $layer, $color, $width, @$_);
  }
}

1;
