package PerlCAD::Command;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use strict;
use Data::DumpEx;

###----------------------------------------------------------------###

register_action({
  name      => 'filenew',
  accel     => 'Ctrl+N',
  menutext  => '&New',
  tooltip   => 'Start a new document',
  whatsthis => '<img src="filenew"> Click this button to open a <b>New</b> file.<br/>If any changes have been made to a file that is currently open, you will be prompted to save them.',
  slot      => 'filenew_slot',
  method    => sub {
    this->slot_process_line('NEW');
  },
});

register_command({
  command => 'NEW',
  method  => sub { this->parent->get_document->new_document; },
});

###----------------------------------------------------------------###

register_action({
  name      => 'fileopen',
  accel     => 'Ctrl+O',
  menutext  => '&Open',
  tooltip   => 'Open an existing file',
  whatsthis => '<img src="fileopen"> Click this button to <b>open</b> an existing file.<br/>If any changes have been made to a file that is currently open, you will be prompted to save them.',
  slot      => 'fileopen_slot',
  method    => sub {
    this->slot_process_line('OPEN');
  },
});

register_command({
  command => 'OPEN',
  has_dialog => 1,
  requires   => sub {
    my $state   = shift;
    my $lineref = shift;
    return this->parent->get_document->open_document_nondialog_requires(this, $state, $lineref);
  },
  method  => sub {
    my $state = shift;
    my $file;
    if (! $state->{use_dialog}) {
      $file = shift || return;
    }
    return this->parent->get_document->open_document($file, $state);
  },
});

###----------------------------------------------------------------###

register_action({
  name      => 'exit',
  accel     => 'Ctrl+Q',
  menutext  => 'E&xit',
  tooltip   => 'Exit the program',
  slot      => 'fileexit_slot',
  method    => sub {
    this->slot_process_line('EXIT');
  },
});

register_command({
  command => 'EXIT',
  aliases => ['QUIT'],
  method  => sub {
    my $state   = shift;
    my $extra   = shift || '';
    my $command = $state->{command};
    my $alias   = $state->{alias};
    my $msg     = "Exiting - $command called";
    $msg .= " via $alias" if $alias;
    $msg .= " - $extra"   if $extra;
    emit_warning($msg);
    this->parent->close;
  },
  requires => [ { type => 'textline', optional => 1, }, ],
});

###----------------------------------------------------------------###

register_action({
  name      => 'filesaveas',
  accel     => '',
  menutext  => 'Save &As ...',
  tooltip   => 'Save File As ...',
  whatsthis => 'Choose this option to <b>Save</b> the current working file <b>As</b> under filename.',
  slot      => 'filesaveas_slot',
  method    => sub {
    this->slot_process_line('SAVEAS');
  },
});

register_command({
  command => 'SAVEAS',
  method  => sub {
    this->parent->get_document->save_document_as;
  },
});

###----------------------------------------------------------------###

register_action({
  name      => 'filesave',
  accel     => 'Ctrl+S',
  menutext  => '&Save',
  tooltip   => 'Save current file',
  whatsthis => '<img src="filesave"> Click this button to <b>Save</b> the current working file.',
  slot      => 'filesave_slot',
  method    => sub {
    this->slot_process_line('SAVE');
  },
});

register_command({
  command => 'SAVE',
  method  => sub { this->parent->get_document->save_document },
});

###----------------------------------------------------------------###

register_action({
  name      => 'fileprint',
  accel     => 'Ctrl+P',
  menutext  => '&Print',
  tooltip   => 'Print',
  whatsthis => '<img src="fileprint"> Click this button to <b>print</b> the current document.',
  slot      => 'fileprint_slot',
  method    => sub {
    this->slot_process_line('PRINT');
  },
});

###----------------------------------------------------------------###

1;
