package PerlCAD::Command;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use strict;
use Data::DumpEx;

###----------------------------------------------------------------###

register_action({
  name      => 'formatlimits',
  menutext  => "Drawing &Limits",
  tooltip   => 'Set the drawing limits',
  whatsthis => '<img src="limits"> Use this command to set the drawing limits.',
  slot      => 'slot_formatlimits',
  method    => sub {
    this->slot_process_line('LIMITS');
  },
});

register_command({
  command => 'LIMITS',
  title   => 'Reset Model space limits:',
  pre     => 'Specifiy lower left corner OR',
  options => {
    OFF  => { key => 'O', method => sub { this->emit_general("Should turn limits on") }, },
    ON   => { key => 'N', method => sub { this->emit_general("Should turn limits off") }, },
    SET  => {
      no_show  => 1,
      requires => [
       { type => 'POINT', msg => 'Lower left corner'  },
       { type => 'POINT', msg => 'Upper right corner' },
      ],
      method   => sub {
        my $state = shift;
        my $p1 = shift;
        my $p2 = shift;
        this->emit_general("Should set limits to (@$p1) - (@$p2)");
      },
    },
  },
  default => {
    option      => 'SET',
    activate_on => 'POINT',
    text        => 'lower left corner',
  },
});

register_action({
  name      => 'formatlweight',
  menutext  => "Line&Weight",
  tooltip   => 'Change the default lineweight',
#  whatsthis => '<img src="limits"> Use this command to set the drawing limits.',
  slot      => 'slot_formatlweight',
  method    => sub {
    this->slot_process_line("'LWEIGHT");
  },
});

register_command({
  command => 'LWEIGHT',
  allow_transparent => 1,
  requires => [{
    type   => 'UINT',
    msg    => sub {
      my ($val, $i) = @{ this->parent->get_line_width };
      $val = sprintf("Qt Lineweight %d",$val) if $val =~ /^\d+$/;
      my $s = "Current lineweight: %d (%s)\nEnter default lineweight for new objects";
      return sprintf($s, $i, $val);
    },
  }],
  method => sub {
    my $state = shift;
    my $width = shift;
    die "Invalid width $width\n" if $width !~ /^(0|[1-9]\d*)$/;
    this->parent->set_line_width($width);
  },
});


register_action({
  name      => 'colorize',
  menutext  => "&Color",
  tooltip   => 'Change the default color',
  slot      => 'slot_colorize',
  method    => sub {
    this->slot_process_line("'COLOR");
  },
});

# variable => 'CECOLOR'
register_command({
  command => 'COLOR',
  allow_transparent => 1,
  has_dialog => 1,
  requires => [{type => 'COLOR'}], # requires is only required for non-dialog
  method => sub {
    my $state = shift;
    my $name;
    if ($state->{use_dialog}) {
      require  PerlCAD::UI::DialogColor;
      import   PerlCAD::UI::DialogColor; # must be imported
      $name = &PerlCAD::UI::DialogColor::get_color(this->parent,
                                                   this->parent->prop_color->colorname,
                                                   this->parent->doc->active_view()->is_black_on_white);
    } else {
      $name = shift;
    }
    return if ! defined $name;
    this->parent->prop_color->set_colorname($name);
  },
});


register_action({
  name      => 'formatlayer',
  menutext  => "&Layer",
  tooltip   => 'Edit layer properties',
  slot      => 'slot_formatlayer',
  method    => sub {
    this->slot_process_line("'LAYER");
  },
});

register_command({
  command => 'LAYER',
  allow_transparent => 1,
  has_dialog => 1,
  requires => [],
  method => sub {
    my $state = shift;
    my $layer;
    if ($state->{use_dialog}) {
      require   PerlCAD::UI::DialogLayer;
      import    PerlCAD::UI::DialogLayer; # must be imported
      $layer = &PerlCAD::UI::DialogLayer::get_layer(this->parent);
    } else {
      dex $state;
      $layer = shift;
    }
    dex $layer;
    return if ! defined $layer;
  },
});

###----------------------------------------------------------------###

1;
