package PerlCAD::Command;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use strict;
use Data::DumpEx;

###----------------------------------------------------------------###

register_action({
  name      => 'viewmenubar',
  accel     => 'Ctrl+M',
  menutext  => 'Show &Menubar',
  tooltip   => 'Toggle the display of the Menu bar',
  whatsthis => '',
  is_toggle => 1,
  slot      => 'viewmenubar_slot',
  method    => sub {
    my $toggle = shift;
    my $on = ($toggle) ? "ON" : "OFF";
    this->slot_process_line("MENUBAR $on");
  },
});

register_command({
  command => 'MENUBAR',
  allow_transparent => 1,
  options => {
    OFF => { key => 'O', method => sub { this->parent->toggle_menubar(0) }, no_recurse => 1 },
    ON  => { key => 'N', method => sub { this->parent->toggle_menubar(1) }, no_recurse => 1 },
  },
  default => sub { get_action('viewmenubar')->isOn ? 'OFF' : 'ON' },
});

###----------------------------------------------------------------###

register_action({
  name      => 'viewstatusbar',
  accel     => 'F10',
  menutext  => 'Show Status&bar',
  tooltip   => 'Toggle the display of the Status bar',
  is_toggle => 1,
  slot      => 'viewstatusbar_slot',
  method    => sub {
    my $toggle = shift;
    my $on = ($toggle) ? "ON" : "OFF";
    this->slot_process_line("STATUSBAR $on");
  },
});

register_command({
  command => 'STATUSBAR',
  allow_transparent => 1,
  options => {
    OFF => { key => 'O', method => sub { this->parent->toggle_statusbar(0) }, no_recurse => 1 },
    ON  => { key => 'N', method => sub { this->parent->toggle_statusbar(1) }, no_recurse => 1 },
  },
  default => sub { get_action('viewstatusbar')->isOn ? 'OFF' : 'ON' },
});

###----------------------------------------------------------------###

register_action({
  name      => 'viewcommandwin',
#  accel     => 'Ctrl+M',
  menutext  => 'Show &Command Window',
  tooltip   => 'Toggle the display of the Command entry window',
  whatsthis => '',
  is_toggle => 1,
  slot      => 'viewcommandwin_slot',
  method    => sub {
    my $toggle = shift;
    my $on = ($toggle) ? "ON" : "OFF";
    this->slot_process_line("COMMANDWIN $on");
  },
});

register_command({
  command => 'COMMANDWIN',
  allow_transparent => 1,
  options => {
    OFF => { key => 'O', method => sub { this->parent->toggle_commandwin(0) }, no_recurse => 1 },
    ON  => { key => 'N', method => sub { this->parent->toggle_commandwin(1) }, no_recurse => 1 },
  },
  default => sub { get_action('viewcommandwin')->isOn ? 'OFF' : 'ON' },
});

###----------------------------------------------------------------###

register_command({
  command => 'TOOLBAR',
  allow_transparent => 1,
  pre     => "Specify toolbar to toggle",
  options => sub {
    my $array = this->parent->toolbars;
    emit_general("Toolbars:");
    my %commands = ();
    foreach my $i (1 .. $#$array) { # register an option for each toolbar
      my $on = $array->[$i]->isVisible ? "ON" : "OFF";
      emit_general("  $i. ".$array->[$i]->label." ($on)");
      $commands{$i} = {
        no_recurse => 1,
        method => sub {
          my $obj = $array->[$i];
          my $new_state = $obj->isVisible ? 0 : 1;
          this->parent->toggle_toolbar($i, $new_state);
          emit_info(sprintf "Toolbar %d \"%s\" %s", $i, $obj->label, ($new_state ? 'ON' : 'OFF'));
        },
      };
    }
    $commands{OFF} = { key => 'O', method => sub { toolbars_on_off($array, 0) }, extra => 'All', no_recurse => 1 };
    $commands{ON}  = { key => 'N', method => sub { toolbars_on_off($array, 1) }, extra => 'All', no_recurse => 1 };
    return \%commands;
  },
});

sub toolbars_on_off {
  my $array = shift;
  my $new_state = shift;
  foreach my $i (1 .. $#$array) {
    my $obj = $array->[$i];
    this->parent->toggle_toolbar($i, $new_state);
    emit_info(sprintf "Toolbar %d \"%s\" %s", $i, $obj->label, ($new_state ? 'ON' : 'OFF'));
  }
}

###----------------------------------------------------------------###

register_command({
  command => 'REGEN',
  aliases => ['REDRAW'],
  method  => sub {
    my $line = shift;
    this->parent->doc->active_view()->reload;
  },
});

###----------------------------------------------------------------###

register_action({
  name      => 'viewgrid',
  accel     => 'F7',
  menutext  => '&Grid',
  tooltip   => 'Display Grid',
  whatsthis => '<img src="viewgrid"> <b>Grid Mode</b><br/>Toggle the display of the onscreen grid.',
  is_toggle => 1,
  slot      => 'viewgrid_slot',
  method    => sub {
    my $toggle = shift;
    my $on = ($toggle) ? "ON" : "OFF";
    this->slot_process_line("'GRID $on");
  },
});

register_command({
  command => 'GRID',
  allow_transparent => 1,
  pre     => 'Specify grid spacing (X) or',
  options => {
    X      => { method => \&set_grid, requires => [{type => 'DISTANCE'}], no_show => 1 },
    OFF    => { key => 'O', method => sub { this->parent->toggle_grid(0) }, no_recurse => 1 },
    ON     => { key => 'N', method => sub { this->parent->toggle_grid(1) }, no_recurse => 1 },
    SNAP   => { key => 'S', method => sub { set_grid(undef,
                                                     this->parent->doc->active_view()->snap_space_x,
                                                     this->parent->doc->active_view()->snap_space_y) } },
    ASPECT => {
      key => 'A',
      method => \&set_grid,
      requires => [{
        type    => 'DISTANCE',
        msg     => 'Specify the horizontal spacing (X)',
        default => sub { this->parent->doc->active_view()->grid_space_x },
      }, {
        type    => 'DISTANCE',
        msg     => 'Specify the horizontal spacing (Y)',
        default => sub { this->parent->doc->active_view()->grid_space_y },
      }]
    },
  },
  default => sub {
    my $text = this->parent->doc->active_view()->grid_space_x;
    return {
      activate_on => 'DISTANCE',
      text   => $text,
      option => 'X',
    };
  },
});

register_action({
  name      => 'viewsnap',
  accel     => 'F9',
  menutext  => '&Snap',
  tooltip   => 'Snap Mode',
  whatsthis => '<img src="viewsnap"> <b>Snap Mode</b><br/>When snap mode is on, the drawing cursor will automatically move to the next snap node.',
  is_toggle => 1,
  slot      => 'viewsnap_slot',
  method    => sub {
    my $toggle = shift;
    my $on = ($toggle) ? "ON" : "OFF";
    this->slot_process_line("'SNAP $on");
  },
});

register_command({
  command => 'ELEV',
  allow_transparent => 1,
  method => sub {},
});

register_command({
  command => 'SNAP',
  aliases => 'SN',
  allow_transparent => 1,
  pre     => 'Specify Snap spacing (X) or',
  options => {
    X      => { method => \&set_snap, requires => [{type => 'DISTANCE'}], no_show => 1 },
    OFF    => { key => 'O', method => sub { this->parent->toggle_snap(0) }, no_recurse => 1 },
    ON     => { key => 'N', method => sub { this->parent->toggle_snap(1) }, no_recurse => 1 },
    GRID   => { key => 'G', method => sub { set_snap(undef,
                                                     this->parent->doc->active_view()->grid_space_x,
                                                     this->parent->doc->active_view()->grid_space_y) } },
    ASPECT => {
      key => 'A',
      method => \&set_snap,
      requires => [{
        type    => 'DISTANCE',
        msg     => 'Specify the horizontal spacing (X)',
        default => sub { this->parent->doc->active_view()->snap_space_x },
      }, {
        type => 'DISTANCE',
        msg  => 'Specify the horizontal spacing (Y)',
        default => sub { this->parent->doc->active_view()->snap_space_y },
      }]
    },
  },
  default => sub {
    my $text = this->parent->doc->active_view()->snap_space_x;
    return {
      activate_on => 'DISTANCE',
      text   => $text,
      option => 'X',
    };
  },
});

sub set_grid {
  my $state = shift;
  my $x     = shift;
  my $y     = shift;
  my $view = this->parent->doc->active_view();
  $y ||= $x / $view->grid_space_x * $view->grid_space_y; # save aspect
  $view->set_grid_space($x, $y);
  $view->update_grid;
  this->emit_info("GRID set to $x, $y");
}

sub set_snap {
  my $state = shift;
  my $x     = shift;
  my $y     = shift;
  my $view = this->parent->doc->active_view();
  $y ||= $x / $view->grid_space_x * $view->grid_space_y; # save aspect
  $view->set_snap_space($x, $y);
  $view->update_crosshairs;
  this->emit_info("SNAP set to $x, $y");
}

###----------------------------------------------------------------###

register_action({
  name      => 'viewortho',
  accel     => 'F8',
  menutext  => '&Ortho',
  tooltip   => 'Display Ortho',
  whatsthis => '<img src="viewortho"> <b>Ortho Mode</b><br/>When ortho mode is on lines will be drawn parallel to the grid.',
  is_toggle => 1,
  slot      => 'viewortho_slot',
  method    => sub {
    my $toggle = shift;
    my $on = ($toggle) ? "ON" : "OFF";
    this->slot_process_line("'ORTHO $on");
  },
});

register_command({
  command => 'ORTHO',
  allow_transparent => 1,
  options => {
    OFF => { key => 'O', method => sub { this->parent->toggle_ortho(0) }, no_recurse => 1 },
    ON  => { key => 'N', method => sub { this->parent->toggle_ortho(1) }, no_recurse => 1 },
  },
  default => sub { get_action('viewortho')->isOn ? 'OFF' : 'ON' },
});

###----------------------------------------------------------------###

#  command->get_action('viewmag')->addTo(this->{mzoom});
#  command->get_action('viewmagfit')->addTo(this->{mzoom});
#  command->get_action('viewmag1')->addTo(this->{mzoom});
#  command->get_action('viewmag+')->addTo(this->{mzoom});
#  command->get_action('viewmag-')->addTo(this->{mzoom});
#  All / Center / Dynamic / Extents / Previous / Scale(X/XP) / Window / Realtime

register_action({
  name      => 'viewmag',
#  accel     => 'F10',
  menutext  => 'Zoom &Window',
  tooltip   => 'Zoom to window',
  slot      => 'viewmag_slot',
  method    => sub {
    this->slot_process_line("'ZOOM WINDOW");
  },
});

register_action({
  name      => 'viewmagfit',
#  accel     => 'F10',
  menutext  => 'Zoom &All',
  tooltip   => 'Zoom to the extents of the drawing',
  slot      => 'viewmagfit_slot',
  method    => sub {
    this->slot_process_line("'ZOOM ALL");
  },
});

register_action({
  name      => 'viewmagx',
#  accel     => 'F10',
  menutext  => 'Zoom Scale (&X)',
  tooltip   => 'Zoom by scale factor',
  slot      => 'viewmagx_slot',
  method    => sub {
    this->slot_process_line("'ZOOM X");
  },
});

register_action({
  name      => 'viewmagxp',
#  accel     => 'F10',
  menutext  => 'Zoom Scale (XP)',
  tooltip   => 'Zoom by scale percentage',
  slot      => 'viewmagxp_slot',
  method    => sub {
    this->slot_process_line("'ZOOM XP");
  },
});

register_action({
  name      => 'viewmag+',
#  accel     => 'F10',
  menutext  => 'Zoom In',
  tooltip   => 'Zoom In',
  slot      => 'viewmagin_slot',
  method    => sub {
    this->slot_process_line("'ZOOM IN");
  },
});

register_action({
  name      => 'viewmag-',
#  accel     => 'F10',
  menutext  => 'Zoom Out',
  tooltip   => 'Zoom Out',
  slot      => 'viewmagout_slot',
  method    => sub {
    this->slot_process_line("'ZOOM OUT");
  },
});

register_action({
  name      => 'viewmagprevious',
#  accel     => 'F10',
  menutext  => 'Zoom &Previous',
  tooltip   => 'Zoom to the previous zoom',
  slot      => 'viewmagprevious_slot',
  method    => sub {
    this->slot_process_line("'ZOOM PREVIOUS");
  },
});

register_action({
  name      => 'reload',
#  accel     => 'F10',
  menutext  => 'Regenerate',
  tooltip   => 'Regenerate the current view',
  slot      => 'regen_slot',
  method    => sub {
    this->slot_process_line("REGEN");
  },
});

register_command({
  command => 'ZOOM',
  allow_transparent => 1,
  aliases => ['Z'],
  pre     => 'Specify corner OR',
  options => {
    ALL       => {
      key     => 'A',
      method  => sub { this->parent->doc->active_view()->zoom_extents },
    },
    IN        => {
      key     => 'I',
      method  => sub { this->parent->doc->active_view()->zoom_in },
    },
    OUT       => {
      key     => 'O',
      method  => sub { this->parent->doc->active_view()->zoom_out },
    },
    PREVIOUS  => {
      key     => 'P',
      method  => sub { this->parent->doc->active_view()->zoom_previous },
    },
    X         => {
      key     => 'X',
      method  => sub { my $state = shift; this->parent->doc->active_view()->zoom_by_x(shift()) },
      requires => [{type => 'REAL', msg => 'Specify zoom factor'}],
    },
    XP        => {
      key     => '',
      method  => sub { my $state = shift; this->parent->doc->active_view()->zoom_by_xp(shift()) },
      requires => [{type => 'REAL', msg => 'Specify zoom percentage'}],
    },
    WINDOW     => {
      key      => 'W',
      requires => [{
        type => 'POINT',
        msg  => 'Specify first corner of zoom window',
      }, {
        type => 'POINT',
        msg  => 'Specify second corner of zoom window',
        draw => \&draw_zoom_window,
      }],
      method  => sub {
        my $state = shift;
        my $p1    = shift;
        my $p2    = shift;
        this->parent->doc->active_view()->zoom($p1->[0], $p1->[1], $p2->[0], $p2->[1]);
      },
    },
  },
  default => {
    activate_on => 'POINT',
    text        => 'first corner',
    option      => 'WINDOW',
  },
});

sub draw_zoom_window {
  my $state = shift;
  my $p1    = shift;
  my $p2    = shift;
  $_->[2] ||= 0 foreach $p1, $p2;
  die "Must be planar" if $p1->[2] != $p2->[2];
  return [['line', $p1, [$p1->[0], $p2->[1], $p1->[2]]],
          ['line', $p2, [$p1->[0], $p2->[1], $p1->[2]]],
          ['line', $p2, [$p2->[0], $p1->[1], $p1->[2]]],
          ['line', $p1, [$p2->[0], $p1->[1], $p1->[2]]],
          ];
}


###----------------------------------------------------------------###

register_action({
  name      => 'view_vpoint',
  menutext  => '3D Views',
  tooltip   => 'Change the current viewing angle',
  slot      => 'vpoint_slot',
  method    => sub {
    this->slot_process_line("'VPOINT");
  },
});

register_command({
  command => 'VPOINT',
  allow_transparent => 1,
  requires => [{
    type => 'POINT',
    msg  => 'Specify view point OR rotation',
  }],
  method => sub {
    my $state = shift;
    my $point = shift;
    this->parent->doc->active_view()->set_vpoint($point);
  },
});

###----------------------------------------------------------------###

register_action({
  name      => 'view_vports',
  menutext  => 'Viewports',
  tooltip   => 'Change the number of view ports',
  slot      => 'vport_slot',
  method    => sub {
    this->slot_process_line("'VPORTS");
  },
});

register_action({
  name      => 'view_vports1',
  menutext  => '1 - Single',
  tooltip   => 'Change to a single view port',
  slot      => 'vport_slot1',
  method    => sub {
    this->slot_process_line("'VPORTS 1");
  },
});

register_action({
  name      => 'view_vports2',
  menutext  => '2 - Double (horizontal)',
  tooltip   => 'Change to a double horizontal view port',
  slot      => 'vport_slot2',
  method    => sub {
    this->slot_process_line("'VPORTS 2");
  },
});

register_action({
  name      => 'view_vports3',
  menutext  => '3 - Triple (two left, one right)',
  tooltip   => 'Change to three view ports',
  slot      => 'vport_slot3',
  method    => sub {
    this->slot_process_line("'VPORTS 3");
  },
});

register_action({
  name      => 'view_vports4',
  menutext  => '4 - 2 x 2 Grid',
  tooltip   => 'Change to a 2 x 2 grid of view ports',
  slot      => 'vport_slot4',
  method    => sub {
    this->slot_process_line("'VPORTS 4");
  },
});


register_command({
  command => 'VPORTS',
  pre     => "Select the number of viewports",
  options => {
    1 => { method => sub { this->parent->set_vports(1) } },
    2 => { method => sub { this->parent->set_vports(2) } },
    3 => { method => sub { this->parent->set_vports(3) } },
    4 => { method => sub { this->parent->set_vports(4) } },
  },
});

###----------------------------------------------------------------###

1;
