package PerlCAD::Config;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use Qt;
use Qt::isa qw(Qt::Object);
use constant APP_NAME => 'perlcad';

use Data::DumpEx;

sub NEW {
  shift->SUPER::NEW(@_[0,1]);

  ### determine the configuration directories
  my @dirs;
  if ($ENV{QTDIR} && $ENV{QTDIR} =~ m|^(.+)/lib/qt|i) {
    push @dirs, $1;
  }
  if (home_dir()) {
    push @dirs, home_dir() . "/.kde";
  }
  this->{_dirs} = \@dirs;

  icon_dirs();
}

sub home_dir {
  return $ENV{HOME} || '';
}

sub dirs {
  return this->{_dirs} || [];
}

### look for directories with icons
sub icon_dirs {
  return this->{_icon_dirs} ||= do {
    my $dirs  = dirs();
    my $theme = get('config/kdeglobals', 'Icons')->{'Theme'};
    my $size  = get('config/kdeglobals', 'MainToolbarIcons')->{'Size'};
    my @DIRS  = ();
    foreach my $dir (@$dirs) {
      $dir .= "/share/icons";
      unshift @DIRS, $dir;
      if ($theme) {
        unshift @DIRS, "$dir/$theme/22x22/actions";
        unshift @DIRS, "$dir/$theme/${size}x${size}/actions" if $size;
      }
    }
    
    @DIRS = grep {-d} @DIRS;
    \@DIRS; # return of the do
  };
}

sub get {
  my ($area, $name) = (@_ == 1) ? ('general', shift()) : (shift(), shift());
  return this->{$area}->{$name} if defined(this->{$area}->{$name});
  return undef if this->{_loaded}->{$area};
  this->{_loaded}->{$area} = 1;

  my $hash = this->{$area} ||= {};
  foreach my $dir (@{ this->{_dirs} }) {
    my $file = "$dir/share/$area";
    next if ! open(IN, $file);
    my $section;
    while (defined(my $line = <IN>)) {
      if ($line =~ m|^\s*\[(.*)\]\s*$|) {
        $section = $1;
        next;
      }
      next if ! $section;
      next if $line =~ m/^\s*\#/;
      next if $line !~ m/^([^=]+)=(.*?)\s*$/;
      $hash->{$section}->{$1} = $2;
    }
    close(IN);
  }

  ### cleanup immutables here

  return $hash->{$name};
}

sub set {
}

###----------------------------------------------------------------###

1;
