package PerlCAD::Icon;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use strict;
use vars qw(%PIXMAPS);

sub data {
  my $name = shift;
  return undef if ! $name || ! $PIXMAPS{$name};
  seek DATA, 0, 0;
  my $found;
  my @data = ();
  while (<DATA>) {
    if ($found) {
      last if /^\s*$/;
      last if /^---/;
      push @data, $_;
    } else {
      next if ! /^---\Q$name\E/;
      $found = 1;
    }
  }
  return \@data;
}

sub write_icons_to_directory {
  my $directory = shift;
  my @list = ();
  foreach my $name (sort keys %PIXMAPS) {
    my $file = "$directory/$name.xpm";
    if (open (OUT, ">$file")) {
      print OUT "/* XPM */\nstatic char *line[]={\n";
      print OUT '"' .join("\",\n\"", @{ data($name) }). '"';
      print OUT "};\n";
      close OUT;
      push @list, $file;
    }
  }
  return \@list;
}

%PIXMAPS = map {$_ => 1} 
qw(
   filenew
   fileopen
   filesave
   fileprint
   undo
   redo
   editcut
   editcopy
   editpaste
   find
   contexthelp
   viewgrid
   viewortho
   viewsnap
   draw_line
   draw_rect
   draw_circle_cr
   draw_circle_2p
   draw_circle_3p
   );

1;

###----------------------------------------------------------------###
__DATA__
'


---filenew------------
22 22 7 1
. c None
# c #000000
b c #2e2e2e
c c #5c5c5c
d c #878787
e c #c2c2c2
a c #ffffff
......................
....##########........
....#aaaaaaa#b#.......
....#aaaaaaa#cb#......
....#aaaaaaa#dcb#.....
....#aaaaaaa#edcb#....
....#aaaaaaa#aedcb#...
....#aaaaaaa#######...
....#aaaaaaaaaaaaa#...
....#aaaaaaaaaaaaa#...
....#aaaaaaaaaaaaa#...
....#aaaaaaaaaaaaa#...
....#aaaaaaaaaaaaa#...
....#aaaaaaaaaaaaa#...
....#aaaaaaaaaaaaa#...
....#aaaaaaaaaaaaa#...
....#aaaaaaaaaaaaa#...
....#aaaaaaaaaaaaa#...
....#aaaaaaaaaaaaa#...
....###############...
......................
......................


---fileopen-----------
22 22 5 1
. c None
# c #000000
c c #848200
a c #ffff00
b c #ffffff
......................
......................
......................
............####....#.
...........#....##.##.
..................###.
.................####.
.####...........#####.
#abab##########.......
#babababababab#.......
#ababababababa#.......
#babababababab#.......
#ababab###############
#babab##cccccccccccc##
#abab##cccccccccccc##.
#bab##cccccccccccc##..
#ab##cccccccccccc##...
#b##cccccccccccc##....
###cccccccccccc##.....
##cccccccccccc##......
###############.......
......................


---filesave-----------
22 22 5 1
. c None
# c #000000
a c #848200
b c #c1c1c1
c c #cab5d1
......................
.####################.
.#aa#bbbbbbbbbbbb#bb#.
.#aa#bbbbbbbbbbbb#bb#.
.#aa#bbbbbbbbbcbb####.
.#aa#bbbccbbbbbbb#aa#.
.#aa#bbbccbbbbbbb#aa#.
.#aa#bbbbbbbbbbbb#aa#.
.#aa#bbbbbbbbbbbb#aa#.
.#aa#bbbbbbbbbbbb#aa#.
.#aa#bbbbbbbbbbbb#aa#.
.#aaa############aaa#.
.#aaaaaaaaaaaaaaaaaa#.
.#aaaaaaaaaaaaaaaaaa#.
.#aaa#############aa#.
.#aaa#########bbb#aa#.
.#aaa#########bbb#aa#.
.#aaa#########bbb#aa#.
.#aaa#########bbb#aa#.
.#aaa#########bbb#aa#.
..##################..
......................


---fileprint----------
22 22 88 2
Qt c None
.2 c #000000
.S c #08ff08
#v c #100810
.U c #101010
#c c #101018
.M c #181018
#e c #181818
.A c #181821
.L c #211821
#l c #212121
.z c #212129
#m c #292129
#u c #292929
#n c #292931
.R c #29ff29
#o c #312931
.T c #313131
#p c #313139
.Z c #31ff31
#q c #393139
#t c #393939
.y c #393942
#s c #423942
.o c #424242
#h c #4a4a52
.n c #5a525a
#r c #5a5a63
.I c #5ace5a
#b c #6b636b
.p c #6b6b6b
.x c #6b6b73
.Y c #6bff63
.l c #736b73
.t c #7b737b
.s c #7b7384
.0 c #7bff7b
.r c #847b84
.u c #847b8c
#g c #84848c
.v c #8c7b94
#i c #8c848c
.w c #8c8494
#j c #8c8c8c
.8 c #8c8c94
.m c #948c94
#k c #948c9c
#f c #949494
.q c #94949c
.J c #94c694
#d c #9c949c
#a c #9c94a5
.k c #9c9c9c
.N c #9c9ca5
.H c #9ccea5
.K c #a59ca5
#. c #a59cad
.i c #a5a5a5
.3 c #a5a5ad
## c #ad9cad
.V c #ada5ad
.d c #adadad
.j c #adadb5
.9 c #b5adb5
.# c #b5b5b5
.a c #bdbdbd
.7 c #bdd6bd
.c c #c6c6c6
.5 c #cec6ce
.b c #cecece
.4 c #ceced6
.F c #d6ced6
.G c #d6cede
.h c #d6d6d6
.E c #d6d6de
.Q c #d6ffd6
.B c #ded6de
.1 c #ded6e7
.g c #dedede
.D c #dedee7
.6 c #e7dee7
.f c #e7e7e7
.C c #e7e7ef
.X c #e7ffe7
.O c #efe7ef
.e c #efefef
.W c #f7f7f7
.P c #ffffff
QtQtQtQtQtQt.#.a.b.b.b.b.c.c.a.a.d.aQtQtQtQt
QtQtQtQtQtQt.a.e.f.f.f.f.f.e.e.e.g.aQtQtQtQt
QtQtQtQtQtQt.a.c.c.c.b.b.c.c.c.c.a.cQtQtQtQt
QtQtQtQtQtQt.#.a.a.a.a.#.a.a.#.#.d.aQtQtQtQt
QtQtQtQtQt.c.d.c.a.c.c.c.a.a.a.c.#QtQtQtQtQt
QtQtQtQtQt.a.a.#.a.a.a.a.a.a.c.c.#QtQtQtQtQt
QtQtQtQtQt.a.#.c.a.a.a.a.a.c.a.c.dQtQtQtQtQt
QtQtQtQtQt.c.a.a.a.a.a.a.a.a.a.a.#QtQtQtQtQt
QtQtQtQtQt.d.b.f.g.g.g.g.g.g.h.g.i.i.jQtQtQt
QtQtQt.a.k.l.#.h.b.h.b.h.b.h.g.g.m.n.o.p.#Qt
QtQt.a.q.r.s.t.t.t.t.t.t.t.u.v.w.x.y.z.A.o.i
Qt.a.k.B.C.D.B.E.E.E.E.F.G.H.I.J.K.o.L.L.M.y
.a.N.O.P.P.P.P.P.P.P.P.P.Q.R.S.R.b.v.T.A.U.L
.V.W.P.P.P.P.P.P.P.P.P.P.X.Y.Z.0.P.1.t.A.2.L
.3.E.4.5.4.h.E.E.g.6.D.B.D.E.7.F.4.5.8.M.2.A
.m.9.j.V.3#..3.K#.#..i#..K#.###a.q.8#b#c.2.L
.m.j.j#..3.K.K.K.N.K.N.N.N.N#a#d#d.w#b#c.2#e
#f#.#..K.N.K.N.N.N#a.k#a#d#d#d#a.m#g#b.M.2#h
.m.3.K.K#a.k#a#d#a.k#a#d#a#d.q.m.8#i.x#c#e.d
#f#g#i.w#j.w#i.8.w#i.8.8.m.8.m#k.8.w#b#e#fQt
.#.l.z.A#l.z#m#m#m#n#o#o#p#p#q#q#p#o#p#fQtQt
QtQt.d#r#s#s#t#p.T.T.T#u#u.z#e#e#v.o.kQtQtQt


---undo---------------
22 22 3 1
. c None
# c #000084
a c #848284
......................
......................
......................
......................
......................
...........######a....
..#......##########...
..##...####......##a..
..###.###.........##..
..######..........##..
..#####...........##..
..######..........##..
..#######.........##..
..########.......##a..
...............a###...
...............###....
......................
......................
......................
......................
......................
......................


---redo---------------
22 22 3 1
. c None
a c #000084
# c #848284
......................
......................
......................
......................
......................
....#aaaaaa...........
...aaaaaaaaaa......a..
..#aa......aaaa...aa..
..aa.........aaa.aaa..
..aa..........aaaaaa..
..aa...........aaaaa..
..aa..........aaaaaa..
..aa.........aaaaaaa..
..#aa.......aaaaaaaa..
...aaa#...............
....aaa...............
......................
......................
......................
......................
......................
......................


---editcut------------
22 22 3 1
. c None
# c #000000
a c #000082
......................
.......#.....#........
.......#.....#........
.......#.....#........
.......#....##........
.......##...#.........
........#...#.........
........##.##.........
.........###..........
.........###..........
..........#...........
.........a#a..........
........aa.aaa........
.......a.a.a..a.......
......a..a.a...a......
.....a...a.a....a.....
....a....a.a....a.....
....a....a..a...a.....
....a....a..a..a......
....a...a....aa.......
.....aaa..............
......................


---editcopy-----------
22 22 6 1
. c None
# c #000000
b c #000082
c c #3c3cfd
d c #8b8bfd
a c #ffffff
......................
......................
########..............
#aaaaaa##.............
#a####a#a#............
#aaaaaa#aa#...........
#a####a#bbbbbbbb......
#aaaaaa#baaaaaabb.....
#a#####aba####abcb....
#aaaaaaabaaaaaabdcb...
#a#####aba####abadcb..
#aaaaaaabaaaaaabbbbbb.
#a#####aba####aaaaaab.
#aaaaaaabaaaaaaaaaaab.
#a#####aba#########ab.
#aaaaaaabaaaaaaaaaaab.
########ba#########ab.
........baaaaaaaaaaab.
........ba#########ab.
........baaaaaaaaaaab.
........bbbbbbbbbbbbb.
......................


---editpaste----------
22 22 8 1
. c None
# c #000000
e c #000084
c c #848200
b c #848284
d c #c6c3c6
a c #ffff00
f c #ffffff
......................
.......#####..........
..######aaa######.....
.######aaaaa######....
##bcb##a###a##bcb##...
#bcb#ddddddddd#bcb#...
#cbc#ddddddddd#cbc#...
#bcb###########bcb#...
#cbcbcbcbcbcbcbcbc#...
#bcbcbcbcbcbcbcbcb#...
#cbcbcbceeeeeeeeee#...
#bcbcbcbefffffffefe...
#cbcbcbcefeeeeefeffe..
#bcbcbcbefffffffefffe.
#cbcbcbcefeeeeefeffffe
#bcbcbcbefffffffeeeeee
#cbcbcbcefeeeeeffffffe
#bcbcbcbeffffffffffffe
#cbcbcbcefeeeeeeeeeefe
.#######effffffffffffe
........eeeeeeeeeeeeee
......................



---find---------------
22 22 106 2
Qt c None
#N c #212121
#J c #292929
#K c #393939
#M c #424242
.m c #5a5a5a
#r c #5a736b
#A c #5a9c84
#B c #636363
.N c #636b6b
#G c #63736b
#v c #63ad8c
#H c #6b6b6b
.c c #6b7373
#D c #6b7b73
.l c #6b7b7b
.W c #6b847b
.t c #6b8c7b
#x c #6b9484
#w c #6bad94
.u c #737373
.v c #737b73
.g c #737b7b
.D c #738c84
.7 c #739484
#d c #73948c
#z c #73a59c
#p c #73b594
#o c #73bd9c
.b c #7b7b7b
.O c #7b8484
#a c #7bad94
#q c #7bad9c
#i c #7bb59c
.5 c #7bbd9c
#b c #7bbda5
#h c #7bc69c
#y c #7bc6a5
.8 c #848484
.4 c #8484b5
.h c #849c94
#n c #84a5ad
.V c #84ada5
#E c #84b59c
#F c #84b5a5
#j c #84bda5
#t c #84c6a5
#l c #84cead
.E c #8c8c8c
#g c #8c8cbd
## c #8c94b5
#m c #8cadbd
#c c #8cb5a5
.L c #8cc6ad
.T c #8ccead
#f c #8cceb5
.a c #949494
.n c #94a59c
#u c #94a5ce
.k c #94bdad
.M c #94c6ad
.6 c #94c6b5
.B c #94ceb5
.1 c #94d6b5
.C c #94d6bd
.9 c #94debd
.K c #9c94d6
.d c #9c9c9c
.w c #9cbdb5
#. c #9cbdd6
#s c #9ccebd
.2 c #9ccece
.0 c #9cdebd
.S c #9ce7bd
.Z c #9ce7c6
.U c #a594de
#L c #a5a5a5
.A c #a5b5de
.3 c #a5b5ef
.I c #a5cece
#k c #a5dec6
.s c #a5e7c6
.f c #adadad
.J c #adadef
.R c #adb5de
.i c #adcec6
.P c #add6c6
.j c #addece
#e c #ade7ce
.# c #b5b5b5
.H c #b5bdde
.Q c #b5dede
.r c #b5efd6
.e c #bdbdbd
.z c #bdc6f7
.y c #bdd6ff
.X c #bde7d6
.q c #bdefde
.Y c #bdf7de
#I c #c6c6c6
.x c #c6def7
.p c #c6f7e7
#C c #ced6ce
.G c #cedeff
.o c #ceffef
.F c #d6fff7
QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt
QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt
QtQtQt.#.a.b.c.c.b.d.eQtQtQtQtQtQtQtQtQtQtQt
QtQt.f.g.h.i.j.j.k.l.b.#QtQtQtQtQtQtQtQtQtQt
Qt.f.m.n.o.o.p.q.r.s.t.u.eQtQtQtQtQtQtQtQtQt
.e.v.w.q.p.x.y.z.A.B.C.D.EQtQtQtQtQtQtQtQtQt
.d.h.F.q.G.H.I.I.J.K.L.M.N.#QtQtQtQtQtQtQtQt
.O.P.o.Q.R.B.S.T.A.U.V.B.W.dQtQtQtQtQtQtQtQt
.g.X.Y.Z.Z.0.1.2.3.4.5.6.7.8QtQtQtQtQtQtQtQt
.g.j.r.0.9.1#..3###a#b#c#d.8QtQtQtQtQtQtQtQt
.v.P#e.1.1#f.A#g.5#h#i#j.W.EQtQtQtQtQtQtQtQt
.E#c#k#f.L#l#m#n#o#p#p#q#r.fQtQtQtQtQtQtQtQt
.f.g#s#t#t#b#u.4#v#w#w#x.u.eQtQtQtQtQtQtQtQt
Qt.E.t#y#b.5#z#d#p#A#A#B.d#C.f.eQtQtQtQtQtQt
Qt.e.b#D#E#F#j#i#a.7.m.dQt.f#B.m.fQtQtQtQtQt
QtQt.e.E.N#D.W.W#G#H.fQtQt#I.b#J#K.E.eQtQtQt
QtQtQtQt.#.d.E.a#L#IQtQtQtQtQt#L#M#J#H.fQtQt
QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt.e.u#N#K.aQt
QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt.d#K#N#H
QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt.##B.m
QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt
QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt


---contexthelp--------
22 22 3 1
. c None
# c #000000
a c #000080
......................
......................
......................
...#........aaaaa.....
...##......aaa.aaa....
...###....aaa...aaa...
...####...aa.....aa...
...#####..aa.....aa...
...######..a....aaa...
...#######.....aaa....
...########...aaa.....
...#########.aaa......
...#####.....aaa......
...##.###.............
...#..###....aaa......
.......###...aaa......
.......###............
........###...........
........###...........
......................
......................
......................


---viewgrid-----------
22 22 5 1
. c None
b c #303030
c c #585858
a c #808080
# c #c3c3c3
......................
......................
..#a...#a...#a...#a...
..ab...ab...ab...cb...
......................
......................
......................
..#a...#a...#a...#a...
..ab...ab...ab...ab...
......................
......................
......................
..#a...#a...#a...#a...
..ab...ab...ab...ab...
......................
......................
......................
..#a...#a...#a...#a...
..ab...ab...ab...ab...
......................
......................
......................


---viewortho----------
22 22 7 1
. c None
d c #0058c0
c c #303030
e c #585858
b c #58a8ff
a c #808080
# c #c3c3c3
......................
......................
..#a...#a...#abbb#abb.
..ac...ac...acdddecdd.
............bd........
............bd........
............bd........
..#a...#a...#a...#a...
..ac...ac...ac...ac...
............bd........
............bd........
............bd........
..#abbb#abbb#a...#a...
..acdddacdddac...ac...
..bd..................
..bd..................
..bd..................
..#a...#a...#a...#a...
..ac...ac...ac...ac...
..bd..................
..bd..................
......................


---viewsnap-----------
22 22 10 1
. c None
f c #000000
b c #303030
c c #585858
a c #808080
h c #a0a0a0
# c #c3c3c3
e c #ff0000
d c #ffc0c0
g c #ffffff
......................
......................
..#a...#a...#a...#a...
..ab...ab...ab...cb...
......................
........d.............
......deed............
..#a..e#ae..#a...#a...
..ab.deaff..ab...ab...
......defgf...........
........fggf..........
........fgggf.........
..#a...#fggggf...#a...
..ab...afgggggf..ab...
........fggggggf......
........fggggcff......
........fggagf........
..#a...#fgffggh..#a...
..ab...aff..agh..ab...
............fggc......
............hcf.......
......................


---draw_line----------
22 22 9 1
. c None
g c #0058c0
d c #303030
e c #585858
f c #58a8ff
b c #808080
c c #a8dcff
a c #c3c3c3
# c #dcdcdc
......................
......................
................#aba..
................a#ab..
...............cbade..
..............cffbea..
.............cffgc....
............cffgc.....
...........cffgc......
..........cffgc.......
.........cffgc........
........cffgc.........
.......cffgc..........
......cffgc...........
.....cffgc............
....cffgc.............
..#abfgc..............
..a#abc...............
..bade................
..abea................
......................
......................


---draw_rect----------
22 22 9 1
. c None
g c #0058c0
d c #303030
e c #585858
f c #58a8ff
b c #808080
c c #a8dcff
a c #c3c3c3
# c #dcdcdc
......................
......................
.................#aba.
..ccccccccccccccca#ab.
..cffffffffffffffbade.
..cfgggggggggggggfbea.
..cfg............cfg..
..cfg............cfg..
..cfg............cfg..
..cfg............cfg..
..cfg............cfg..
..cfg............cfg..
..cfg............cfg..
..cfg............cfg..
..cfg............cfg..
..cfg............cfg..
.#abacccccccccccccfg..
.a#abffffffffffffffg..
.badeggggggggggggggg..
.fbea.................
......................
......................


---draw_circle_cr-----
22 22 9 1
. c None
b c #0058c0
f c #303030
g c #585858
a c #58a8ff
e c #808080
# c #a8dcff
d c #c3c3c3
c c #dcdcdc
......................
.......#aaaaa#........
.....#aaabbbbaa#......
....#aabb####abaa#....
...#abba.......aba#...
..#aba#.........abb...
..aab#...........#b#..
.#aba.............ab..
.aab..............#ba.
.ab#.....cded....cded.
.ab#.....dcde....dcde.
.ab#.....edfg....edfg.
.ab#.....degd....degb.
.#aa.............#aab.
..ab.............#aba.
..#aa...........#aab..
...ab..........##aba..
...#aba......##aaab...
....#bba#####aaabb....
......#bbaaaaabba.....
........abbbba........
......................


---draw_circle_2p-----
22 22 9 1
. c None
b c #0058c0
f c #303030
g c #585858
a c #58a8ff
e c #808080
# c #a8dcff
d c #c3c3c3
c c #dcdcdc
......................
.......#aaaaa#........
.....#aaabbbbaa#......
....#aabb####abaa#....
...#abba.......aba#...
..#aba#.........abb...
..aab#...........#b#..
.#aba.............ab..
.aab..............#ba.
cded.............cded.
dcde.............dcde.
edfg.............edfg.
degd.............degb.
.#aa.............#aab.
..ab.............#aba.
..#aa...........#aab..
...ab..........##aba..
...#aba......##aaab...
....#bba#####aaabb....
......#bbaaaaabba.....
........abbbba........
......................


---draw_circle_3p-----
22 22 9 1
. c None
b c #0058c0
f c #303030
g c #585858
a c #58a8ff
e c #808080
# c #a8dcff
d c #c3c3c3
c c #dcdcdc
......................
.......#aaaaa#........
.....#aaabbbbaa#......
...cdedbb####abaa#....
...dcdea.......aba#...
..#edfg.........abb...
..adegd..........#b#..
.#aba.............ab..
.aab..............#ba.
.ab#.............cded.
.ab#.............dcde.
.ab#.............edfg.
.ab#.............degb.
.#aa.............#aab.
..ab.............#aba.
..#cded.........#aab..
...dcde........##aba..
...edfg......##aaab...
...degda#####aaabb....
......#bbaaaaabba.....
........abbbba........
......................


----------------------
