package PerlCAD::Item::Arc;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use strict;
use Qt;
use Qt::isa qw(Qt::CanvasPolygonalItem);
use Qt::attributes qw(
                      id
                      radius
                      angle1
                      angle2
                      points
                      a_points
                      );
use Data::DumpEx;
use Math::Trig ();

sub NEW {
  my $class = shift;
  my $canvas = shift;
  $class->SUPER::NEW($canvas);

  return if $#_ == -1;

  id = shift;

  ### allow initialize of all
  local this->{no_update} = 1;
  setX(shift);
  setY(shift);
  setRadius(shift || 0);
  setAngles(shift || 0, shift || 360);
  update_points();
}

sub DESTROY {
  hide();
}

sub setX {
  SUPER->setX(@_);
  update_points() if ! this->{no_update};
}

sub setY {
  SUPER->setY(@_);
  update_points() if ! this->{no_update};
}

sub setRadius {
  radius = abs(shift || 0);
  update_points() if ! this->{no_update};
}

sub setAngles {
  my $ang1 = shift;
  my $ang2 = shift;
  foreach ($ang1, $ang2) {
    die "Invalid angle $_" if $ang1 < 0;
    die "Invalid angle $_" if $ang2 > 360;
  }
  die "Angles may not match ($ang1, $ang2)" if $ang1 == $ang2;
  angle1 = &Math::Trig::deg2rad($ang1);
  angle2 = ($ang2 == 360) ? &Math::Trig::pi2() : &Math::Trig::deg2rad($ang2);
}

sub set_all {
  my $p1   = shift;
  my $rad  = shift;
  my $ang1 = shift;
  my $ang2 = shift;
  this->{no_update} = 1;
  setX($p1->x);
  setY($p1->y);
  setRadius($rad);
  setAngles($ang1, $ang2);
  delete this->{no_update};
  update_points();
}

sub startPoint {
  return Qt::Point(this->x, this->y);
}

sub update_points {
  invalidate();
  points   = get_points(radius);
  a_points = get_points(radius + 10);
  update();
}

sub get_points {
  my $r = shift;

  ### optimize the number of points per circle
  ### I would use the makeEllipse or makeArc - but they use way too many points
  ### plus they don't close completely
  ### plus QCanvasEllipse doesn't paint the outline
  # points->makeEllipse(this->x - radius, this->y - radius, radius * 2, radius * 2);
  # points->makeArc(this->x - radius, this->y - radius, radius * 2, radius * 2, 0, 360 * 16);
  my $n = ($r < 5) ? 8 : ($r < 12) ? 16 : ($r < 25) ? 32 : ($r < 75) ? 48 : ($r < 500) ? 64 : 128;
  my @x;
  my @y;
  for (0..$n) {
    my $a = &Math::Trig::pi2() * ($_ / $n) + angle1;
    $a = angle2 if $a > angle2;
    push @x, this->x + $r * cos($a);
    push @y, this->y - $r * sin($a);
    last if $a == angle2;
  }
  my $pa = Qt::PointArray($#x + 1);
  $pa->setPoint($_, $x[$_], $y[$_]) for 0 .. $#x;
  return $pa;
}

sub rtti {
  1001;
}

sub drawShape {
  my $painter = shift;
  $painter->drawPolyline(points);
}

sub areaPoints {
  return a_points;
}

sub boundingRect {
  return points->boundingRect;
}

sub confirm_collision {
  my $x = shift;
  my $y = shift;
  my $offset = shift;
  my $r = sqrt( (x() - $x)**2 + (&y() - $y)**2 );
  return (radius <= $r + $offset * 1.4) ? 1 : 0;
}

1;
