package PerlCAD::Main;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use vars qw($VERSION);
use strict;

$VERSION = '0.0';

BEGIN {
  use constant HAS_QT => eval { require Qt };

  if (HAS_QT) {
    require PerlCAD::MainWindow;
    import PerlCAD::MainWindow;
  }

}

sub main {
  if (! HAS_QT) {
    die "PerlQt missing or improperly installed - cannot continue\n";
  }

  
  my $app = Qt::Application(\@ARGV);
  my $win = PerlCAD::MainWindow();

  $win->show;
  $app->setMainWidget($win);
  $win->load_first_doc;

  exit $app->exec;
}

###----------------------------------------------------------------###

sub import {
  my $pkg  = shift;
  my $cpkg = caller(0);
  no strict 'refs';
  *{"${cpkg}::${_}"} = \&{"${pkg}::${_}"} foreach qw(main);
  use strict 'refs';
}

###----------------------------------------------------------------###

1;


__END__

=head1 NAME

perlcad - The PerlCAD Qt based drafting system

=head1 SYNOPSIS

  From the command line type:

  perlcad

  (The perlcad executable is installed in the bin directory of
   your OS - eg. /usr/bin/perlcad)

  From Perl:

  #!/usr/bin/perl -w

  use strict;
  use PerlCAD::Main;
  PerlCAD::Main->main;
  exit;

  Or from both:

  perl -MPerlCAD::Main -e main

  For a list of command line options, use the --help option, see the
  manpage for perlcad, or see the online documentation (which should
  exist as soon as there is something substantial to document).

  For a list of PerlCAD modules and internals - read on.

=head1 DESCRIPTION

PerlCAD is a PerlQt based CAD / drafting program.
It is intended to be scriptable and extensible and have most of
the features found in AutoCAD (rendering and solid modelling
will not be added until future releases - 2.0 at least - but they
are planned).

=head1 MODULES

The main modules to worry about are.

L<PerlCAD::MainWindow>
  
L<PerlCAD::Doc>

L<PerlCAD::View>

L<PerlCAD::Command>

All other modules will have documentation as time permits.  If
all else fails - view the source.  

=head1 BUGS

Using a scripting language to do ui intensive work.  This could also
be thought of as a feature.

=head1 LICENSE

GNU GPL - see the included file named LICENSE.  GNU GPL is used rather
than the Perl Artistic as the Qt libraries linked to are GPL.

=head1 AUTHOR

Paul Seamons [perlcad] [@] [seamons.com]

=cut
