package PerlCAD::UI::ColorButton;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use strict;

use Qt;
use Qt::isa qw(Qt::Label);
use Qt::attributes qw(
                      pressed
                      mouse_over
                      is_disabled
                      has_focus
                      color
                      );
use Qt::signals
  activated => [],
  ;
use Data::DumpEx;

sub NEW {
  my $class  = shift;
  my $parent = shift;
  my $name   = shift;
  $class->SUPER::NEW($parent,$name);

  color = Qt::black();

  setMinimumSize(Qt::Size(20, 20));
  setPaletteBackgroundColor(color);
  setFrameShape(&Qt::Label::Panel());
  setFrameShadow(&Qt::Label::Sunken());
  setLineWidth(1);
  setFocusPolicy(Qt::Widget::StrongFocus());

}

sub set_size {
  setMinimumSize(shift);
}

sub set_color {
  color = shift;
  setPaletteBackgroundColor(color);
}

sub set_enabled {
  is_disabled = ! shift;
  setFocusPolicy(is_disabled ? Qt::Widget::NoFocus() : Qt::Widget::StrongFocus());
}

###----------------------------------------------------------------###
### make this label think its a button

sub focusInEvent {
  return if is_disabled;
  has_focus = 1;
  setFrameShadow(&Qt::Label::Plain());
}

sub focusOutEvent {
  return if is_disabled;
  has_focus = 0;
  setFrameShadow(&Qt::Label::Sunken());
}

sub enterEvent {
  return if is_disabled;
  mouse_over = 1;
}

sub leaveEvent {
  return if is_disabled;
  setFrameShadow(has_focus ? &Qt::Label::Plain() : &Qt::Label::Sunken() ) if pressed;
  pressed = 0;
  mouse_over = 0;
}

sub keyPressEvent {
  return if is_disabled;
  my $event = shift;
  
  if ($event->key == Qt::Key_Space() || $event->key == Qt::Key_Return()) {
    return mousePressEvent();
  }

  SUPER->keyPressEvent($event);
}

sub keyReleaseEvent {
  return if is_disabled;
  my $event = shift;
  
  if ($event->key == Qt::Key_Space() || $event->key == Qt::Key_Return()) {
    return mouseReleaseEvent();
  }

  SUPER->keyReleaseEvent($event);
}

sub mousePressEvent {
  return if is_disabled;
  my $event = shift;
  
  return if $event && $event->button == Qt::RightButton();

  setFrameShadow(&Qt::Label::Raised());
  pressed = 1;
}

sub mouseReleaseEvent {
  return if is_disabled;
  my $event = shift;

  return if $event && $event->button == Qt::RightButton();

  pressed = 0;
  setFrameShadow(has_focus ? &Qt::Label::Plain() : &Qt::Label::Sunken());

  emit activated();
}

###----------------------------------------------------------------###

1;
