package PerlCAD::UI::ColorCombo;


use strict;
use Qt;
use Qt::isa qw(Qt::ComboBox);
use Qt::attributes qw(
                      colorname
                      mainwindow
                      );
use Qt::signals
  clicked => [],
  ;

use PerlCAD::Color qw(aci_to_name);
use Data::DumpEx;

sub NEW {
  my $class  = shift;
  my $parent = shift;
  my $name   = shift;
  my $window = shift;
  $class->SUPER::NEW($parent, $name);

  mainwindow = $window;

  insertItem(Qt::Pixmap(15,11),"ByLayer", 0);
}

sub set_colorname {
  my $color = colorname = shift;

  ### verify colorname
  if ($color =~ /^(ByLayer|ByBlock)$/i) {
    colorname = (lc($1) eq 'bylayer') ? 'ByLayer' : 'ByBlock';
    $color = mainwindow->prop_layer->color;
  }

  ### cleanup the color
  if ($color =~ /^\d+$/) {
    die "Invalid ACI range ($color)" if $color < 1 || $color > 255;
    $color = aci_to_name($color);
  } elsif ($color =~ /^(-?\d+),(-?\d+),(-?\d+)$/) {
    my ($R, $G, $B) = ($1, $2, $3);
    foreach ($R, $G, $B) {
      $_ = 0 if $_ < 0;
      $_ = 255 if $_ > 255;
    }
    $color = colorname = sprintf("#%02X%02X%02X", $R, $G, $B);
  } elsif ($color =~ /^\#[a-f0-9]{6}$/i) {
    colorname = uc($color);
  } else {
    die "Unknown color option ($color)";
  }

  my $pix = Qt::Pixmap(15,11);
  my $col = Qt::Color();
  $col->setNamedColor($color);
  $pix->fill($col);
  
  changeItem($pix, colorname, 0);
  setCurrentItem(0);
}

sub mousePressEvent {
  my $event = shift;
  if ($event->button == Qt::RightButton()) {
    SUPER->mousePressEvent($event);
    return;
  }
  emit clicked;
}

1;
