package PerlCAD::UI::DialogColor;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use strict;

use Qt;
use Qt::isa qw(Qt::Dialog);
use Qt::slots
  slot_button_bylayer_released  => [],
  slot_button_byblock_released  => [],
  slot_button_select_released   => [],
  slot_checkbox_convert_clicked => [],
  slot_text_changed             => ['const QString&'],
  ;
use Qt::attributes qw(
    button_bylayer
    button_byblock
    button_select
    checkbox_convert
    current_label
    current_value_le
    current_value_la
    button_ok
    button_cancel
    black_on_white
    labels
    groups
);

use PerlCAD::Color qw(name_to_aci aci_to_name int_to_aci
                      aci_to_int name_to_int int_to_name
                      @ACI %ACI);
use PerlCAD::UI::ColorButton;

use Data::DumpEx;

###----------------------------------------------------------------###

sub get_color {
  my $parent = shift;
  my $color  = shift || 'ByLayer';
  my $b_on_w = shift;
  my $rgb    = ($color =~ /^\d+$/) ? $ACI[$color] : eval { name_to_int($color) } || [0,0,0];

  my $dialog = PerlCAD::UI::DialogColor(this->parent, "Color Selection", 1);
  $dialog->set_current($color);
  $dialog->set_black_on_white($b_on_w);

  my $status = $dialog->exec;
  return undef if ! defined($status) || $status != Qt::Dialog::Accepted();

  my $value = $dialog->current_value_le->text();
  return $value;
}

sub NEW {
  shift->SUPER::NEW(@_[0..3]);

  setCaption("Select Color");
  setSizeGripEnabled(1);

  my $l_widget = this; #Qt::Widget(this, '$l_widget');
  $l_widget->setGeometry(Qt::Rect(0, 0, 370, 305));
  my $vbox_outer = Qt::VBoxLayout($l_widget, 11, 6, '$vbox_outer');
  
  my $labels = labels = [];
  my $groups = groups = [];
  my $layout;
  my $s;

  ### start row 1
  my $row1_hbox = Qt::HBoxLayout(undef, 0, 6, '$row1_hbox');
  $vbox_outer->addLayout($row1_hbox);

  ### setup the standard group
  my $group = Qt::GroupBox($l_widget, "standard group");
  $group->setTitle("Standard Color");
  &Qt::WhatsThis::add($group, "Select from the standard color names.");
  $group->setColumnLayout(0, &Vertical());
  $group->layout->setSpacing(3);
  $layout = Qt::HBoxLayout($group->layout);
  $layout->setAlignment(&AlignTop());
  $row1_hbox->addWidget($group);

  ### add standard items
  for my $i (1 .. 9) {
    my $rgb = aci_to_int($i);
    $rgb = [0,0,0] if $i == 7 && black_on_white;
    my $w   = PerlCAD::UI::ColorButton($group, "label_aci_$i");
    $w->set_color(Qt::Color(@$rgb));
    $layout->addWidget($w);

    my $slot = "slot_label_aci_$i";
    import Qt::slots $slot => [];
    this->connect($w, SIGNAL('activated()'), SLOT("${slot}()"));
    no strict 'refs';
    *{__PACKAGE__ . "::" . $slot} = sub {
      set_current($i);
    } if ! defined(*{__PACKAGE__ . "::" . $slot});
    use strict 'refs';
  }

  $row1_hbox->addItem(Qt::SpacerItem(20, 20));



  ### start row 2
  my $row2_hbox = Qt::HBoxLayout(undef, 0, 6, '$row2_hbox');
  $vbox_outer->addLayout($row2_hbox);

  ### setup the gray group
  $group = Qt::GroupBox($l_widget, "gray group");
  $group->setTitle("Gray Shade");
  &Qt::WhatsThis::add($group, "Select from the standard ACI gray shades.");
  $group->setColumnLayout(0, &Vertical());
  $group->layout->setSpacing(3);
  $layout = Qt::HBoxLayout($group->layout);
  $layout->setAlignment(&AlignTop());
  $row2_hbox->addWidget($group);

  ### add gray items
  for my $i (250 .. 255) {
    my $rgb = aci_to_int($i);
    my $w   = PerlCAD::UI::ColorButton($group, "label_aci_$i");
    $w->set_color(Qt::Color(@$rgb));
    $layout->addWidget($w);

    my $slot = "slot_label_aci_$i";
    import Qt::slots $slot => [];
    this->connect($w, SIGNAL('activated()'), SLOT("${slot}()"));
    no strict 'refs';
    *{__PACKAGE__ . "::" . $slot} = sub {
      current_value_le->setText($i);
      current_value_la->set_color(Qt::Color(@$rgb));
    } if ! defined *{__PACKAGE__ . "::" . $slot};
    use strict 'refs';
  }

#  $row2_hbox->addItem(Qt::SpacerItem(0, 20));

  ### setup the logical group
  $group = Qt::GroupBox($l_widget, "logical group");
  $group->setTitle("Logical Color");
  $group->setColumnLayout(0, &Vertical());
  $group->layout->setSpacing(3);
  $layout = Qt::HBoxLayout($group->layout);
  $layout->setAlignment(&AlignTop());
  $row2_hbox->addWidget($group);

  ### add buttons to the row
  button_bylayer = Qt::PushButton("By&Layer", $group, "button_bylayer");
  $s = "Set the default color to the current layer's color.";
  &Qt::WhatsThis::add(button_bylayer, $s);
  $layout->addWidget(button_bylayer);
  this->connect(button_bylayer, SIGNAL('released()'), SLOT('slot_button_bylayer_released()'));

  button_byblock = Qt::PushButton("By&Block", $group, "button_byblock");
  $s = "Set the default color to the object's block's color.";
  &Qt::WhatsThis::add(button_bylayer, $s);
  $layout->addWidget(button_byblock);
  this->connect(button_byblock, SIGNAL('released()'), SLOT('slot_button_byblock_released()'));



  ### start row 3
  my $row3_hbox = Qt::HBoxLayout(undef, 0, 6, '$row3_hbox');
  $vbox_outer->addLayout($row3_hbox);
  
  ### setup the logical group
  $group = Qt::GroupBox($l_widget, "true color group");
  $group->setTitle("True Color");
  &Qt::WhatsThis::add($group, "Select a custom color.");
  $group->setColumnLayout(0, &Vertical());
  $group->layout->setSpacing($group->insideMargin);
  $layout = Qt::HBoxLayout($group->layout);
  $layout->setAlignment(&AlignTop());
  $row3_hbox->addWidget($group);

  ### add the items
  button_select = Qt::PushButton($group, "button_select");
  button_select->setText("&Select Custom Color");
  $s = "Click this button to choose a custom true color from the color selector.";
  &Qt::WhatsThis::add(button_select, $s);
  $layout->addWidget(button_select);
  this->connect(button_select, SIGNAL('released()'), SLOT('slot_button_select_released()'));

  checkbox_convert = Qt::CheckBox($group, "checkbox_convert");
  checkbox_convert->setText("Convert Color to ACI");
  $s = "Checking this box will automatically convert a true color to the ACI (Color Index) value.";
  &Qt::WhatsThis::add(checkbox_convert, $s);
  $layout->addWidget(checkbox_convert);
  this->connect(checkbox_convert, SIGNAL('clicked()'), SLOT('slot_checkbox_convert_clicked()'));


  ### start row 4
  my $row4_hbox = Qt::HBoxLayout(undef, 0, 6, '$row4_hbox');
  $vbox_outer->addLayout($row4_hbox);

  ### add items
  current_label = Qt::Label("&Color:", $l_widget, "color_label");
  $row4_hbox->addWidget(current_label);

  current_value_le = Qt::LineEdit($l_widget, "current_value_le");
  $row4_hbox->addWidget(current_value_le);
  &Qt::WhatsThis::add(current_value_le, "Diplays the current color choice.");
  current_label->setBuddy(this->current_value_le);
  current_value_le->setValidator(&PerlCAD::Color::validator(current_value_le));
  this->connect(current_value_le,SIGNAL("textChanged(const QString&)"), SLOT("slot_text_changed(const QString&)"));

  current_value_la = PerlCAD::UI::ColorButton($l_widget, "current_value_la");
  &Qt::WhatsThis::add(current_value_la, "Diplays the current color choice.");
  current_value_la->set_size(Qt::Size(60,60));
  #current_value_la->set_enabled(0);
  this->connect(current_value_la, SIGNAL('activated()'), SLOT('slot_button_select_released()'));
  $row4_hbox->addWidget(current_value_la);

  $row4_hbox->addItem(Qt::SpacerItem(20, 20));



  ### spacer row 5
  $vbox_outer->addItem(Qt::SpacerItem(20, 20));



  ### start row 6
  my $row6_hbox = Qt::HBoxLayout(undef, 0, 6, '$row6_hbox');
  $vbox_outer->addLayout($row6_hbox);
  $row6_hbox->addItem(Qt::SpacerItem(20,20));

  button_ok = Qt::PushButton($l_widget, "button_ok");
  button_ok->setText("&OK");
  button_ok->setAutoDefault(1);
  button_ok->setDefault(1);
  $row6_hbox->addWidget(button_ok);

  button_cancel = Qt::PushButton($l_widget, "button_cancel");
  button_cancel->setText("&Cancel");
#  button_cancel->setAutoDefault( 1 );
  $row6_hbox->addWidget(button_cancel);

  my $resize = Qt::Size(370, 305);
  $resize = $resize->expandedTo(minimumSizeHint());
  resize( $resize );
#  clearWState( &Qt::WState_Polished );

  Qt::Object::connect(button_ok, SIGNAL "clicked()", this, SLOT "accept()");
  Qt::Object::connect(button_cancel, SIGNAL "clicked()", this, SLOT "reject()");
}

sub set_black_on_white {
  black_on_white = shift;
}

sub set_current {
  my $text  = shift;
  current_value_le->setText($text);
  current_value_le->setFocus;
}

sub slot_text_changed {
  my $text = uc(shift || '');
  my $n = 0;
  my $status = current_value_le->validator->validate($text, $n);
  if ($status == Qt::Validator::Acceptable()) {
    my $name;
    if ($text eq 'BYLAYER' || $text eq 'BYBLOCK') {
      $name = 'black';
    } elsif ($ACI{$text}) {
      my $ind = $ACI{$text};
      $name = int_to_name(@{ $ACI[$ind] });
      $name = black_on_white ? 'black' : 'white' if $ind == 7;
    } elsif ($text =~ /^\d+$/) {
      $name = aci_to_name($text);
    } elsif ($text =~ /^(-\d+),(-\d+),(-\d+)$/) {
      my ($R, $G, $B) = ($1, $2, $3);
      foreach ($R, $G, $B) {
        $_ = 0 if $_ < 0;
        $_ = 255 if $_ > 255;
      }
      $name = sprintf("#%02X%02X%02X", $R, $G, $B);
    } else {
      $name = $text;
    }
    my $color = Qt::Color();
    $color->setNamedColor($name);
    current_value_la->set_color($color);
  }
}

sub slot_button_bylayer_released {
  set_current("ByLayer");
}

sub slot_button_byblock_released {
  set_current("ByBlock");
}

sub slot_button_select_released {
  my $color = current_value_la->color;
  my $name;

  $color = Qt::ColorDialog::getColor($color || Qt::white(), this);
  if (! $color->isValid) {
    current_value_le->setFocus;
    return;
  }

  if (checkbox_convert->isChecked) {
    $name  = int_to_aci($color->red, $color->green, $color->blue);
  } else {
    $name = uc($color->name);
  }

  set_current($name);

  return 1;
}

sub slot_checkbox_convert_clicked {
  return if ! checkbox_convert->isChecked;
  my $val = current_value_le->text;
  my $ind = eval { name_to_aci($val) } || 7;
  if ($ind == 0 || $ind == 256) {
    $ind = $ACI[$ind];
  }
  set_current($ind);
}

###----------------------------------------------------------------###

1;
