package PerlCAD::Units;

###----------------------------------------------------------------###
#  Copyright 2003 - Paul Seamons                                     #
#  Distributed under the GNU General Public License without warranty #
###----------------------------------------------------------------###

use vars qw(%UNIT %FACT);

%UNIT = (
         NONE   => '',
         NM     => "nm",
         UM,    => "\xb5m",
         MM,    => "mm",
         CM,    => "cm",
         METER, => "meter",
         KM,    => "km",
         INCH,  => "inch",
         FOOT,  => "feet",
         ARCH,  => "feet",
         MILE,  => "mile",
         );
%FACT = (
         NONE   => 1.0,
         NM     => 0.000000001,
         UM,    => 0.000001,
         MM,    => 0.001,
         CM,    => 0.01,
         METER, => 1.0,
         KM,    => 1000.0,
         INCH,  => 0.0254,
         FOOT,  => 0.3048,
         ARCH,  => 0.3048,
         MILE,  => 1609.344,
         );    

sub ucs_to_unit {
  my $num     = shift || 0;
  my $unit    = shift || return '';
  my $sig_fig = shift;

  if ($unit eq 'ARCH') {
    my $str   = ($num < 0) ? '-' : '';
    my $feet  = abs($num);
    my $denom = 64;
    my $inch  = $feet - int($feet) * 12;
    my $frac  = $inch = int($inch) * $denom;

    $str .= int($num) . "' ";
    $str .= int($inch);

    if ($frac != 0) {
      while ($frac % 2 == 0 && $denom > 2){
        $frac  /= 2;
        $denom /= 2;
      }
      $str .= '-'.int($frac) .'/'. int($denom);
    }
    $str .= '"';

    return $str;

  } else {
    my $dig = $sig_fig;
    my $str = sprintf("%.${dig}f", $num);
    $str .= "'" if $unit eq 'FOOT';
    $str .= '"' if $unit eq 'INCH';
    return $str;
  }
}

1;
